-- ==============================================================
-- FILE : PLB2DDR_WR.VHD
-- ==============================================================
--
-- This file is part of the PLB Power PC to DDR interface for the
-- HERON-FPGA9.
--
-- This file is derived from Xilinx provided example code.
--
-- ==============================================================
--
--  Date        : 16/09/2005
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : PLB to DDR interface write logic
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    16-09-05    First Written
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.std_logic_unsigned.all;

-- pragma translate_off
library UNISIM;
  use UNISIM.vcomponents.all;
-- pragma translate_on


entity PLB2DDR_WR is
  port (
    Bus2IP_UserClk   : in  std_logic;
    Bus2IP_UserRst   : in  std_logic;
    Bus2IP_Data      : in  std_logic_vector(0 to 63);
    Bus2IP_Addr      : in  std_logic_vector(0 to 31);
    Bus2IP_BE        : in  std_logic_vector(0 to 7);
    Bus2IP_Burst     : in  std_logic;
    Bus2IP_CE        : in  std_logic_vector(0 to 0);
    Bus2IP_CS        : in  std_logic_vector(0 to 0);
    Bus2IP_RNW       : in  std_logic;
    Bus2IP_WrCE      : in  std_logic_vector(0 to 0);
    Bus2IP_WrReq     : in  std_logic;
    IP2Bus_WrAddrAck : out std_logic;
    IP2Bus_WrAck     : out std_logic;
    WR_DATA          : out std_logic_vector(71 downto 0);
    WR_ADDR          : out std_logic_vector(24 downto 0);
    WR_ADDR_WEN      : out std_logic;
    WR_ADDR_FF       : in  std_logic;
    WR_ADDR_AF       : in  std_logic;
    WR_RISE_WEN      : out std_logic;
    WR_RISE_FF       : in  std_logic;
    WR_RISE_AF       : in  std_logic;
    WR_FALL_WEN      : out std_logic;
    WR_FALL_FF       : in  std_logic;
    WR_FALL_AF       : in  std_logic;
    WR_DM_WEN        : out std_logic;
    WR_DM_FF         : in  std_logic;
    WR_DM_AF         : in  std_logic
  );
end PLB2DDR_WR;


architecture RTL of PLB2DDR_WR is

  signal Wr_Addr_Reg          : std_logic_vector(24 downto 0);
  signal Write_BE             : std_logic_vector(0 to 7);
  signal Write_Enable         : std_logic;
  signal Write_Allow          : std_logic;
  signal Write_Addr_Toggle    : std_logic;
  signal Write_DelayedReq     : std_logic;
  signal Write_CompleteSingle : std_logic;

begin

  IP2Bus_WrAddrAck <= Write_Enable;
  IP2Bus_WrAck     <= Write_Enable;

  WR_ADDR <= Wr_Addr_Reg;

  WR_ADDR_WEN <= (Write_Enable AND Write_Addr_Toggle) OR Write_CompleteSingle;

  WR_RISE_WEN <= Write_Enable OR Write_CompleteSingle;
  WR_FALL_WEN <= Write_Enable OR Write_CompleteSingle;
  WR_DM_WEN   <= Write_Enable OR Write_CompleteSingle;

  Write_Enable <= Write_Allow AND (Bus2IP_WrReq OR Write_DelayedReq);

  Write_Allow <= (NOT WR_ADDR_AF) AND (NOT WR_RISE_AF) AND (NOT WR_FALL_AF) AND (NOT WR_DM_AF);

  process(Bus2IP_UserRst, Bus2IP_UserClk)
  begin
    if Bus2IP_UserRst='1' then
      Write_DelayedReq <= '0';
    elsif rising_edge(Bus2IP_UserClk) then
      Write_DelayedReq <= '0';
      if (Bus2IP_WrReq='1'     and Write_Allow='0') or
         (Write_DelayedReq='1' and Write_Allow='0') then
        Write_DelayedReq <= '1';
      end if;
    end if;
  end process;

  Write_CompleteSingle <= Write_Addr_Toggle AND (NOT Bus2IP_CE(0));

  process(Bus2IP_UserRst, Bus2IP_UserClk)
  begin
    if Bus2IP_UserRst='1' then
      Write_Addr_Toggle <= '0';
    elsif rising_edge(Bus2IP_UserClk) then
      Write_Addr_Toggle <= '0';
      if (Write_Addr_Toggle='0' and Write_Enable='1' and Bus2IP_CE(0)='1') or
         (Write_Addr_Toggle='1' and Write_Enable='0' and Bus2IP_CE(0)='1') then
        Write_Addr_Toggle <= '1';
      end if;
    end if;
  end process;

  process(Bus2IP_UserRst, Bus2IP_UserClk)
  begin
    if Bus2IP_UserRst='1' then
      Wr_Addr_Reg <= (others=>'0');
    elsif rising_edge(Bus2IP_UserClk) then
      if Write_Enable='1' and Write_Addr_Toggle='0' then
        Wr_Addr_Reg <= Bus2IP_Addr(5 to 29);
      end if;
    end if;
  end process;

  Write_BE(7) <= Bus2IP_BE(7) when Bus2IP_Addr(29)='0' else '0';
  Write_BE(6) <= Bus2IP_BE(6) when Bus2IP_Addr(29)='0' else '0';
  Write_BE(5) <= Bus2IP_BE(5) when Bus2IP_Addr(29)='0' else '0';
  Write_BE(4) <= Bus2IP_BE(4) when Bus2IP_Addr(29)='0' else '0';
  Write_BE(3) <= Bus2IP_BE(3) when Bus2IP_Addr(29)='0' else Bus2IP_BE(7);
  Write_BE(2) <= Bus2IP_BE(2) when Bus2IP_Addr(29)='0' else Bus2IP_BE(6);
  Write_BE(1) <= Bus2IP_BE(1) when Bus2IP_Addr(29)='0' else Bus2IP_BE(5);
  Write_BE(0) <= Bus2IP_BE(0) when Bus2IP_Addr(29)='0' else Bus2IP_BE(4);

  WR_DATA(0) <= (NOT Write_BE(7)) OR Write_CompleteSingle;
  WR_DATA(1) <= (NOT Write_BE(6)) OR Write_CompleteSingle;
  WR_DATA(2) <= (NOT Write_BE(5)) OR Write_CompleteSingle;
  WR_DATA(3) <= (NOT Write_BE(4)) OR Write_CompleteSingle;
  WR_DATA(4) <= (NOT Write_BE(3)) OR Write_CompleteSingle;
  WR_DATA(5) <= (NOT Write_BE(2)) OR Write_CompleteSingle;
  WR_DATA(6) <= (NOT Write_BE(1)) OR Write_CompleteSingle;
  WR_DATA(7) <= (NOT Write_BE(0)) OR Write_CompleteSingle;

  WR_DATA(39 downto  8) <= Bus2IP_Data(0 to 31) when Bus2IP_Addr(29)='0' else Bus2IP_Data(32 to 63);
  WR_DATA(71 downto 40) <= Bus2IP_Data(32 to 63);

end RTL;
