--------------------------------------------------------------------------------
-- FILE : OPB2LED.VHD
--------------------------------------------------------------------------------
--
-- This file is part of the PowerPC Example A.
--
--------------------------------------------------------------------------------
--
--  Date        : 14/09/2007
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : OPB General Purpose IO - LED peripheral
--
--------------------------------------------------------------------------------
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    14-09-07    First Written for EDK9.1, based on Xilinx
--                                     example code - OPB GPIO v3.01.a.
--
--------------------------------------------------------------------------------
--
--  Structure:
--
--       opb_gpio.vhd
--          \ opb_ipif_ssp1.vhd
--          \ user_logic.vhd
--          \ pwm.vhd
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
  use ieee.std_logic_1164.all;

library ipif_common_v1_00_c;
  use ipif_common_v1_00_c.ipif_pkg.all;

library opb2led; 
  use opb2led.all;

entity opb2led is  
  generic
  (
    C_BASEADDR          : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_HIGHADDR          : std_logic_vector(0 to 31) := X"00000000";
    C_USER_ID_CODE      : integer                   := 3;
    C_OPB_AWIDTH        : integer                   := 32;
    C_OPB_DWIDTH        : integer                   := 32;
    C_FAMILY            : string                    := "virtex2"; -- not used
    C_INTERRUPT_PRESENT : INTEGER := 0
  );
  port
  (
    OPB_ABus    : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE      : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_Clk     : in  std_logic;
    OPB_DBus    : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW     : in  std_logic;
    OPB_Rst     : in  std_logic;
    OPB_select  : in  std_logic;
    OPB_seqAddr : in  std_logic;
    Sln_DBus    : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    Sln_errAck  : out std_logic;
    Sln_retry   : out std_logic;
    Sln_toutSup : out std_logic;
    Sln_xferAck : out std_logic;

    led         : out STD_LOGIC_VECTOR(0 to 3)

  );

  --fan-out attributes for XST
  attribute MAX_FANOUT            : string;
  attribute MAX_FANOUT of OPB_Clk : signal is "10000";
  attribute MAX_FANOUT of OPB_Rst : signal is "10000";

  attribute MIN_SIZE : string ;
  attribute MIN_SIZE of C_BASEADDR: constant is "0x200";
  attribute SIGIS : string ;
  attribute SIGIS of OPB_Clk: signal is "Clk";
  attribute SIGIS of OPB_Rst: signal is "Rst";
  attribute ADDR_TYPE : string ;
  attribute ADDR_TYPE of C_BASEADDR : constant is "REGISTER";
  attribute ADDR_TYPE of C_HIGHADDR : constant is "REGISTER";
  attribute ASSIGNMENT : string ;
  attribute ASSIGNMENT of C_BASEADDR : constant is "REQUIRE";
  attribute ASSIGNMENT of C_HIGHADDR : constant is "REQUIRE";

end entity opb2led;

-------------------------------------------------------------------------------
-- Architecture Section
-------------------------------------------------------------------------------

architecture imp of opb2led is 

-------------------------------------------------------------------------------
-- Constant Declarations
-------------------------------------------------------------------------------
constant PIPELINE_MODEL : integer := 5; -- example user logic uses 
                                        -- OPB-In and OPB-Out pipeline regs

constant NUM_INTR       : integer := 2;

constant INTR_TYPE      : integer := INTR_POS_EDGE_DETECT;

constant INTR_BASEADDR  : std_logic_vector(0 to 31)  := C_BASEADDR or X"00000100";
constant INTR_HIGHADDR  : std_logic_vector(0 to 31)  := C_BASEADDR or X"000001FF";
constant GPIO_HIGHADDR  : std_logic_vector(0 to 31)  := C_BASEADDR or X"0000000F";

-------------------------------------------------------------------------------
-- Signal and Type Declarations
-------------------------------------------------------------------------------
--IP Interconnect (IPIC) signal list --do not delete
signal Bus2IP_Addr        : std_logic_vector(0 to C_OPB_AWIDTH-1);
signal Bus2IP_BE          : std_logic_vector(0 to C_OPB_DWIDTH/8-1);
signal Bus2IP_Burst       : std_logic;
signal Bus2IP_Clk         : std_logic;
signal Bus2IP_CS          : std_logic;
signal Bus2IP_Data        : std_logic_vector(0 to C_OPB_DWIDTH-1);
signal Bus2IP_RdCE        : std_logic;
signal Bus2IP_Reset       : std_logic;
signal Bus2IP_RNW         : std_logic;
signal Bus2IP_WrCE        : std_logic;
signal IP2Bus_Ack         : std_logic;
signal IP2Bus_Data        : std_logic_vector(0 to C_OPB_DWIDTH-1);
signal IP2Bus_Error       : std_logic;
signal IP2Bus_PostedWrInh : std_logic;
signal IP2Bus_Retry       : std_logic;
signal IP2Bus_ToutSup     : std_logic;
signal IP2Bus_Intr        : std_logic_vector(0 to NUM_INTR-1); 
signal Bus2IP_Data_i      : std_logic_vector(0 to C_OPB_DWIDTH-1);

component user_logic is
  port
  (
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Clk   : in  std_logic;
    Bus2IP_CS    : in  std_logic;
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RdCE  : in  std_logic;
    Bus2IP_Reset : in  std_logic;
    Bus2IP_WrCE  : in  std_logic;
    IP2Bus_Ack   : out std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    LED          : out std_logic_vector(0 to 3)
  );
end component user_logic;

------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------

IP2Bus_Intr <= (others=>'0');

OPB_IPIF_SSP1_I : entity opb2led.opb_ipif_ssp1
  generic map
  (
    C_BASEADDR         => C_BASEADDR,
    C_HIGHADDR         => GPIO_HIGHADDR,
    C_INTR_BASEADDR    => INTR_BASEADDR,
    C_INTR_HIGHADDR    => INTR_HIGHADDR,
    C_USER_ID_CODE     => C_USER_ID_CODE,
    C_PIPELINE_MODEL   => PIPELINE_MODEL,
    C_NUM_USER_INTR    => NUM_INTR, 
    C_INCLUDE_INTR     => C_INTERRUPT_PRESENT /= 0, 
    C_USER_INTR_TYPE   => INTR_TYPE,
    C_OPB_AWIDTH       => C_OPB_AWIDTH,
    C_OPB_DWIDTH       => C_OPB_DWIDTH,
    C_FAMILY           => C_FAMILY
  )
  port map
  (
    OPB_ABus           => OPB_ABus,
    OPB_BE             => OPB_BE,
    OPB_Clk            => OPB_Clk,
    OPB_DBus           => OPB_DBus,
    OPB_RNW            => OPB_RNW,
    OPB_Rst            => OPB_Rst,
    OPB_select         => OPB_select,
    OPB_seqAddr        => OPB_seqAddr,
    Sln_DBus           => Sln_DBus,
    Sln_errAck         => Sln_errAck,
    Sln_retry          => Sln_retry,
    Sln_toutSup        => Sln_toutSup,
    Sln_xferAck        => Sln_xferAck,
    Bus2IP_Addr        => Bus2IP_Addr,
    Bus2IP_BE          => Bus2IP_BE,
    Bus2IP_Burst       => Bus2IP_Burst,
    Bus2IP_Clk         => Bus2IP_Clk,
    Bus2IP_CS          => Bus2IP_CS,
    Bus2IP_Data        => Bus2IP_Data,
    Bus2IP_RdCE        => Bus2IP_RdCE,
    Bus2IP_Reset       => Bus2IP_Reset,
    Bus2IP_RNW         => Bus2IP_RNW,
    Bus2IP_WrCE        => Bus2IP_WrCE,
    IP2Bus_Ack         => IP2Bus_Ack,
    IP2Bus_Data        => IP2Bus_Data,
    IP2Bus_Error       => IP2Bus_Error,
    IP2Bus_PostedWrInh => IP2Bus_PostedWrInh,
    IP2Bus_Retry       => IP2Bus_Retry,
    IP2Bus_ToutSup     => IP2Bus_ToutSup,
    IP2Bus_Intr        => IP2Bus_Intr,
    IP2INTC_Irpt       => open );

USER_LOGIC_I : user_logic
  port map
  (
    Bus2IP_Addr  => Bus2IP_Addr,
    Bus2IP_Clk   => Bus2IP_Clk,
    Bus2IP_CS    => Bus2IP_CS,
    Bus2IP_Data  => Bus2IP_Data,
    Bus2IP_Reset => Bus2IP_Reset,
    Bus2IP_RdCE  => Bus2IP_RdCE,
    Bus2IP_WrCE  => Bus2IP_WrCE,
    IP2Bus_Ack   => IP2Bus_Ack,
    IP2Bus_Data  => IP2Bus_Data,
    LED          => led );

IP2Bus_Retry       <= '0'; -- no retry
IP2Bus_Error       <= '0'; -- no error
IP2Bus_ToutSup     <= '0'; 
-- no timeout suppress. If IP2Bus_Ack is delayed more than 15 clocks, drive
-- IP2Bus_ToutSup to '1' to avoid arbiter timeout on OPB.
IP2Bus_PostedWrInh <= '0'; -- do not inhibit posted write

end architecture imp;
