--------------------------------------------------------------------------------
-- FILE : USER_LOGIC.VHD
--------------------------------------------------------------------------------
--
-- This file is part of the PowerPC Example A.
--
--------------------------------------------------------------------------------
--
--  Date        : 14/09/2007
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : OPB General Purpose IO - LED peripheral
--
--------------------------------------------------------------------------------
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    14-09-07    First Written for EDK9.1, based on Xilinx
--                                     example code - OPB GPIO v3.01.a.
--
--------------------------------------------------------------------------------
--
--  Structure:
--
--       opb_gpio.vhd
--          \ opb_ipif_ssp1.vhd
--          \ user_logic.vhd
--          \ pwm.vhd
--
-------------------------------------------------------------------------------


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.std_logic_signed.all;
  use IEEE.std_logic_misc.all;


library Unisim;
  use Unisim.all;


entity user_logic is
  port (
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Clk   : in  std_logic;
    Bus2IP_CS    : in  std_logic;
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RdCE  : in  std_logic;
    Bus2IP_Reset : in  std_logic;
    Bus2IP_WrCE  : in  std_logic;
    IP2Bus_Ack   : out std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    LED          : out std_logic_vector(0 to 3)
  );
end entity user_logic;


architecture RTL of user_logic is

  component PWM is
    port (
      PWM_Out : out std_logic;
      Clk     : in std_logic;
      Control : in std_logic_vector(0 to 7)
    );
  end component PWM;

  signal RdCE_rg            : std_logic;

  signal addr               : std_logic_vector(0 to 2);

  signal led0_control_reg   : std_logic_vector(0 to 7);
  signal led1_control_reg   : std_logic_vector(0 to 7);
  signal led2_control_reg   : std_logic_vector(0 to 7);
  signal led3_control_reg   : std_logic_vector(0 to 7);

  signal led0_control       : std_logic_vector(0 to 7);
  signal led1_control       : std_logic_vector(0 to 7);
  signal led2_control       : std_logic_vector(0 to 7);
  signal led3_control       : std_logic_vector(0 to 7);

  signal led_sig0           : std_logic;
  signal led_sig1           : std_logic;
  signal led_sig2           : std_logic;
  signal led_sig3           : std_logic;

  signal led_invert_control : std_logic_vector(0 to 3);

begin

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      RdCE_rg <= Bus2IP_RdCE;
    end if;
  end process;

  IP2Bus_Ack <= Bus2IP_WrCE or RdCE_rg;

  IP2Bus_Data(0 to 23) <= (others=>'0');

  -- OPB Address - Big Endian
  addr <= Bus2IP_Addr(27 to 29);

  process (addr,led0_control_reg,led1_control_reg,
                led2_control_reg,led3_control_reg,led_invert_control) is
  begin
    case addr is
      when "000"  => IP2Bus_Data(24 to 31) <= led0_control_reg;
      when "001"  => IP2Bus_Data(24 to 31) <= led1_control_reg;
      when "010"  => IP2Bus_Data(24 to 31) <= led2_control_reg;
      when "011"  => IP2Bus_Data(24 to 31) <= led3_control_reg;
      when "100"  => IP2Bus_Data(24 to 31) <= "0000" & led_invert_control;
      when others => IP2Bus_Data(24 to 31) <= "0000" & led_invert_control;
    end case;
  end process;

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      if Bus2IP_Reset='1' then
        led0_control_reg   <= (others=>'0'); 
        led1_control_reg   <= (others=>'0'); 
        led2_control_reg   <= (others=>'0'); 
        led3_control_reg   <= (others=>'0'); 
        led_invert_control <= (others=>'0');
      else
        if (Bus2IP_CS='1' and Bus2IP_WrCE='1') then
          case addr is
            when "000"  => led0_control_reg   <= Bus2IP_Data(24 to 31);
            when "001"  => led1_control_reg   <= Bus2IP_Data(24 to 31);
            when "010"  => led2_control_reg   <= Bus2IP_Data(24 to 31);
            when "011"  => led3_control_reg   <= Bus2IP_Data(24 to 31);
            when "100"  => led_invert_control <= Bus2IP_Data(28 to 31);
            when others => null;
          end case;
        end if;
      end if;
    end if;
  end process;

  -- PWM instantiations
  -- Zero LSB of Control since differences in this bit are not visible in LEDs
  led0_control <= led0_control_reg(0 to 6) & '0';
  led1_control <= led1_control_reg(0 to 6) & '0';
  led2_control <= led2_control_reg(0 to 6) & '0';
  led3_control <= led3_control_reg(0 to 6) & '0';

  PWM_0 : PWM port map (PWM_Out=>led_sig0, Clk=>Bus2IP_Clk, Control=>led0_control);
  PWM_1 : PWM port map (PWM_Out=>led_sig1, Clk=>Bus2IP_Clk, Control=>led1_control);
  PWM_2 : PWM port map (PWM_Out=>led_sig2, Clk=>Bus2IP_Clk, Control=>led2_control);
  PWM_3 : PWM port map (PWM_Out=>led_sig3, Clk=>Bus2IP_Clk, Control=>led3_control);

  -- LED outputs
  LED(0) <= led_sig0 XOR led_invert_control(0);
  LED(1) <= led_sig1 XOR led_invert_control(1);
  LED(2) <= led_sig2 XOR led_invert_control(2);
  LED(3) <= led_sig3 XOR led_invert_control(3);

end architecture RTL;
