/*****************************************************************************/
/*																			 */
/*	PPC_FFT_Ex.c															 */
/*																			 */
/*	This program demonstrates how to interface a function implemented in	 */
/*	FPGA gates with C code running on the Power PC.							 */
/*																			 */
/*	The program runs in a continuous loop, where the processor fills data	 */
/*	into a block RAM, requests the external FPGA 'hardware processing unit'	 */
/*	to process that data, and then waits for completion of that operation.	 */
/*																			 */
/*	In this example a CoreGen FFT function block has been used by the FPGA	 */
/*	processing unit, but many other functions may be substituted.			 */
/*																			 */
/*	Ver     Modified By      Date      Changes								 */
/*	---     -----------      ----      -------								 */
/*	1.0     R. Williams    01-11-04    First Written						 */
/*																			 */
/*****************************************************************************/


/* Uncomment the following line to use interrupts */
//#define USE_INTERRUPTS

/* Uncomment the following line to use cache */
//#define USE_CACHE


/* Xilinx provided header file for interrupt functions */
#ifdef USE_INTERRUPTS
#include "xexception_l.h"
#endif

/* Xilinx provided header file for cache functions */
#ifdef USE_CACHE
#include "xcache_l.h"
#endif


/* Pointer to FFT Hardware Processing Unit */
volatile unsigned int *fft_hpu  = (unsigned int *)0x40000000;

/* Pointer to Block RAM interfacing data to FFT engine */
volatile unsigned int *fft_bram = (unsigned int *)0x80000000;


#ifdef USE_INTERRUPTS
void ConnectToProcessor();
void DeviceDriverHandler(void *CallbackRef);
#endif


/* Shared variable used by main thread and interrupt service routine */
#ifdef USE_INTERRUPTS
volatile static Xboolean InterruptProcessed = XFALSE;
#endif


int main(void)
{
	int i;
	unsigned int fft_done;


#ifdef USE_CACHE
	XCache_EnableICache((unsigned int)0x01);
	XCache_EnableDCache((unsigned int)0x01);
	XCache_WriteCCR0((unsigned int)0x00700a00);
#endif


#ifdef USE_INTERRUPTS
    /* This step is processor specific, connect the handler for the 
       interrupt controller to the interrupt source for the processor */
    ConnectToProcessor();

	/* Enable Hardware Processing Unit interrupt */
	fft_hpu[1] = 0x01;
#endif


	while (1) {

		// Write sine-wave into DSOCM Block RAM
		for (i=0;i<8;i++) {
			fft_bram[(i*32) + 0]  = 0x00000000;
			fft_bram[(i*32) + 1]  = 0x000018F9;
			fft_bram[(i*32) + 2]  = 0x000030FB;
			fft_bram[(i*32) + 3]  = 0x0000471C;
			fft_bram[(i*32) + 4]  = 0x00005A82;
			fft_bram[(i*32) + 5]  = 0x00006A6D;
			fft_bram[(i*32) + 6]  = 0x00007641;
			fft_bram[(i*32) + 7]  = 0x00007D89;
			fft_bram[(i*32) + 8]  = 0x00007FFF;
			fft_bram[(i*32) + 9]  = 0x00007D89;
			fft_bram[(i*32) + 10] = 0x00007641;
			fft_bram[(i*32) + 11] = 0x00006A6D;
			fft_bram[(i*32) + 12] = 0x00005A82;
			fft_bram[(i*32) + 13] = 0x0000471C;
			fft_bram[(i*32) + 14] = 0x000030FB;
			fft_bram[(i*32) + 15] = 0x000018F9;
			fft_bram[(i*32) + 16] = 0x00000000;
			fft_bram[(i*32) + 17] = 0x0000E707;
			fft_bram[(i*32) + 18] = 0x0000CF05;
			fft_bram[(i*32) + 19] = 0x0000B8E4;
			fft_bram[(i*32) + 20] = 0x0000A57E;
			fft_bram[(i*32) + 21] = 0x00009593;
			fft_bram[(i*32) + 22] = 0x000089BF;
			fft_bram[(i*32) + 23] = 0x00008277;
			fft_bram[(i*32) + 24] = 0x00008001;
			fft_bram[(i*32) + 25] = 0x00008277;
			fft_bram[(i*32) + 26] = 0x000089BF;
			fft_bram[(i*32) + 27] = 0x00009593;
			fft_bram[(i*32) + 28] = 0x0000A57E;
			fft_bram[(i*32) + 29] = 0x0000B8E4;
			fft_bram[(i*32) + 30] = 0x0000CF05;
			fft_bram[(i*32) + 31] = 0x0000E707;
		}

		// Start FFT 'Hardware Processing Unit'
		fft_hpu[0] = 0x00;

#ifdef USE_INTERRUPTS
	    /* Wait for the interrupt to be processed */
		while (InterruptProcessed == XFALSE);
	    InterruptProcessed = XFALSE;
#else
	    /* Wait until the Hardware Processing Unit flags completion */
		do fft_done = *fft_hpu;
		while ((fft_done & 0x02) == 0x02);
#endif
	}

	return(0);
}


#ifdef USE_INTERRUPTS
void ConnectToProcessor()
{
    /*
     * Initialize the PPC405 exception table
     */
    XExc_Init();

    /*
     * Register the interrupt controller handler with the exception table
     */
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,
                         (XExceptionHandler)DeviceDriverHandler,
                         (void *)0);

    /*
     * Enable non-critical exceptions
     */
    XExc_mEnableExceptions(XEXC_NON_CRITICAL);
}
#endif


#ifdef USE_INTERRUPTS
void DeviceDriverHandler(void *CallbackRef)
{  
    /* Indicate the interrupt has been processed */
	fft_hpu[2] = 0x00;

    InterruptProcessed = XTRUE;
}
#endif
