-- ==============================================================
-- FILE : USER_AP1.VHD  -  MUST be modified by the USER
-- ==============================================================
--
-- This file is part of the FFT Example, for the HERON-FPGA12
-- module.
--
-- This file must be modified by the user.
--
-- ==============================================================
--
--  Date        : 11/11/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : User code section
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    01-11-04    First Written
--  1.1     R. Williams    01-08-05    Changes made to DDR memory
--                                     interface to increase the
--                                     performance when reading.
--  2.0     R. Williams    16-09-05    Added PowerPC to DDR memory
--                                     interface.
--  2.1     R. Williams    09-06-06    Minor fix to HE_RD_6F logic.
--                                     Processor access sequencing
--                                     fix to HE_DDR logic.
--
-- ==============================================================



-- ==============================================================
--   Package : CONFIG
-- ==============================================================
--
-- This package allow users to set up the different clock domains
-- of the application (FIFO interfaces) in the top level.
-- Please refer to the HUNT documentation for more details.
--
-- ==============================================================

package CONFIG is

  -- FIFO CLOCK
  -- ==========

--   --------------------------------------------------------------------
--   Normally you will use the same clock source for the input and output
--   HERON FIFOs. In that case you need a single clock domain.
--   Then you drive SRC_FCLK_G with the frequency you want, which for the 
--   HEPC9/HECPCI9/HERON_BASE2 must be >=60Mhz and <= 100Mhz.
--   The clock you use in your design is then FCLK_G which has the corrct 
--   phase relative to the FIFO data.
--   In this case set FCLK_G_DOMAIN = TRUE

--   If however you want to use different clocks for the input and output 
--   HERON FIFOs, you have seperate clock domains.
--   Now you drive the signals SRC_FCLK_RD and SRC_FCLK_WR with the frequencies
--   you want, (still probably >60Mhz and <100Mhz) and your design must use 
--   FCLK_RD to deal with input FIFO data but FCLK_WR to deal with output FIFO data.
--   In this case set FCLK_G_DOMAIN = FALSE

  -- To use a single common FIFO clock, set FCLK_G_DOMAIN = TRUE
  constant FCLK_G_DOMAIN : boolean := TRUE;

end CONFIG;


-- ==============================================================
--   Module : USER_AP
-- ==============================================================

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;
  use IEEE.numeric_std.all;
  use WORK.CONFIG.all;


--pragma translate_off
library sim_lib;
  use sim_lib.all;

library UNISIM;
  use UNISIM.VCOMPONENTS.all;
--pragma translate_on


entity USER_AP is
  port (
    RESET             : in  std_logic;                      -- Asynchronous reset active high
    CONFIG            : in  std_logic;                      -- System wide Config signal
-- CLOCK IO
    OSC0              : in  std_logic;                      -- Xtal OSC0
    OSC3              : in  std_logic;                      -- Xtal OSC3 factory default fitted with 100Mhz
-- MISCELLANEOUS IO
    UDPRES            : out std_logic;                      -- reset the carrier (high if NOT used)
    LED               : out std_logic_vector(4 downto 0);   -- LEDs
    MOD_HAS_PROC      : out std_logic;
    CID               : in  std_logic_vector(3 downto 0);   -- Carrier ID
    MID               : in  std_logic_vector(3 downto 0);   -- Module ID
-- UMIs
    UMI_IN            : in  std_logic_vector(3 downto 0);
    UMI_OUT           : out std_logic_vector(3 downto 0);
    UMI_EN            : out std_logic_vector(3 downto 0);
-- READ FIFO CLK
    FCLK_RD           : in  std_logic;                      -- FIFO_IN clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_RD       : out std_logic;                      -- FIFO_IN clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- WRITE FIFO CLK
    FCLK_WR           : in  std_logic;                      -- FIFO_OUT clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_WR       : out std_logic;                      -- FIFO_OUT clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- COMMON FIFO CLK
    FCLK_G            : in  std_logic;                      -- Common FIFO clock to be used in this module (only when FCLK_G_DOMAIN=TRUE)
    SRC_FCLK_G        : out std_logic;                      -- Common FIFO clock source for the top level  (only when FCLK_G_DOMAIN=TRUE)
-- FIFO IN / HE_RD_6F interface
    INFIFO_READ_REQ   : out std_logic_vector(5 downto 0);   -- FIFO_IN Read Request
    INFIFO_DVALID     : in  std_logic_vector(5 downto 0);   -- FIFO_IN Data Valid
    INFIFO_SINGLE     : in  std_logic_vector(5 downto 0);   -- FIFO_IN Single data item available
    INFIFO_BURST      : in  std_logic_vector(5 downto 0);   -- FIFO_IN Block of data available
    INFIFO0_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 0
    INFIFO1_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 1
    INFIFO2_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 2
    INFIFO3_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 3
    INFIFO4_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 4
    INFIFO5_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 5
-- FIFO OUT / HE_WR_6F interface
    OUTFIFO_READY     : in  std_logic_vector(5 downto 0);   -- FIFO_OUT Ready Flag
    OUTFIFO_WRITE     : out std_logic_vector(5 downto 0);   -- FIFO_OUT Write Command
    OUTFIFO_D         : out std_logic_vector(31 downto 0);  -- FIFO_OUT Data
-- SERIAL BUS / HE_USER Message interface
    MSG_CLK           : out std_logic;                      -- HSB clock source for the top level
    MSG_DIN           : in  std_logic_vector(7 downto 0);   -- data received from HSB
    MSG_ADDR          : in  std_logic_vector(7 downto 0);   -- HSB address
    MSG_AEN           : in  std_logic;                      -- address enable
    MSG_WEN           : in  std_logic;                      -- write access
    MSG_REN           : in  std_logic;                      -- read access
    MSG_DONE          : in  std_logic;                      -- message successfully transmitted
    MSG_COUNT         : in  std_logic;                      -- user's counter enable
    MSG_CLEAR         : in  std_logic;                      -- user's counter clear
    MSG_READY         : out std_logic;                      -- to acknowledge an access
    MSG_SEND_MSG      : out std_logic;                      -- message send command
    MSG_CE            : out std_logic;                      -- to control speed operation
    MSG_DOUT          : out std_logic_vector(7 downto 0);   -- data sent to HSB
    MSG_SEND_ID       : out std_logic;                      -- ID information send command
    MSG_LAST_BYTE     : out std_logic;                      -- to indicate when the last byte is presented
-- CONNECTOR DIGITAL IO
    CONN_A_IN         : in  std_logic_vector(29 downto 0);  -- Data input on Connector A
    CONN_A_OUT        : out std_logic_vector(29 downto 0);  -- Data output on Connector A
    CONN_A_EN         : out std_logic_vector(29 downto 0);  -- Output enables for Connector A
    CONN_B_IN         : in  std_logic_vector(29 downto 0);  -- Data input on Connector B
    CONN_B_OUT        : out std_logic_vector(29 downto 0);  -- Data output on Connector B
    CONN_B_EN         : out std_logic_vector(29 downto 0);  -- Output enables for Connector B
-- FLASH interface
    FLASH_ADDR        : out std_logic_vector(23 downto 0) ; -- FLASH Address
    FLASH_IN          : in  std_logic_vector(7 downto 0);   -- FLASH Data-In
    FLASH_OUT         : out std_logic_vector(7 downto 0);   -- FLASH Data-Out
    FLASH_EN          : out std_logic_vector(7 downto 0);   -- FLASH Data-Out Enable
    FLASH_CE          : out std_logic;                      -- FLASH Chip Enable
    FLASH_WE          : out std_logic;                      -- FLASH Write Enable
    FLASH_OE          : out std_logic;                      -- FLASH Output Enable
    FLASH_RP          : out std_logic;                      -- FLASH Reset/Power-Down
    FLASH_VPEN        : out std_logic;                      -- FLASH Program Enable
    FLASH_STS         : in  std_logic;                      -- FLASH Status
-- DDR User Clock A
    DDR_A_USER_CLK    : out std_logic;                      -- DDR Port A Clock source
-- DDR User Reset A
    DDR_A_USER_RST    : out std_logic;                      -- DDR Port A Data Count Reset
-- DDR Data Count
    DDR_A_DATA_COUNT  : in  std_logic_vector(31 downto 0);  -- DDR Port A Data Count
-- DDR Write Port A
    DDR_A_WR_DATA     : out std_logic_vector(71 downto 0);  -- DDR Port A Write Data
    DDR_A_WR_ADDR     : out std_logic_vector(24 downto 0);  -- DDR Port A Write Address
    DDR_A_WR_ADDR_WEN : out std_logic;                      -- DDR Port A Wr-Address Enable
    DDR_A_WR_ADDR_FF  : in  std_logic;                      -- DDR Port A Wr-Address Full Flag
    DDR_A_WR_ADDR_AF  : in  std_logic;                      -- DDR Port A Wr-Address Almost Full
    DDR_A_WR_RISE_WEN : out std_logic;                      -- DDR Port A Rising Data Enable
    DDR_A_WR_RISE_FF  : in  std_logic;                      -- DDR Port A Rising Data Full Flag
    DDR_A_WR_RISE_AF  : in  std_logic;                      -- DDR Port A Rising Data Almost Full
    DDR_A_WR_FALL_WEN : out std_logic;                      -- DDR Port A Falling Data Enable
    DDR_A_WR_FALL_FF  : in  std_logic;                      -- DDR Port A Falling Data Full Flag
    DDR_A_WR_FALL_AF  : in  std_logic;                      -- DDR Port A Falling Data Almost Full
    DDR_A_WR_DM_WEN   : out std_logic;                      -- DDR Port A Data Mask Enable
    DDR_A_WR_DM_FF    : in  std_logic;                      -- DDR Port A Data Mask Full Flag
    DDR_A_WR_DM_AF    : in  std_logic;                      -- DDR Port A Data Mask Almost Full
-- DDR Read Port A
    DDR_A_RD_DATA     : in  std_logic_vector(63 downto 0) ; -- DDR Port A Read Data
    DDR_A_RD_ADDR     : out std_logic_vector(24 downto 0) ; -- DDR Port A Read Address
    DDR_A_RD_ADDR_WEN : out std_logic;                      -- DDR Port A Rd-Address Enable
    DDR_A_RD_ADDR_FF  : in  std_logic;                      -- DDR Port A Rd-Address Full Flag
    DDR_A_RD_ADDR_AF  : in  std_logic;                      -- DDR Port A Rd-Address Almost Full
    DDR_A_RD_RISE_REN : out std_logic;                      -- DDR Port A Rising Data Enable
    DDR_A_RD_RISE_EF  : in  std_logic;                      -- DDR Port A Rising Data Empty Flag
    DDR_A_RD_RISE_AE  : in  std_logic;                      -- DDR Port A Rising Data Almost Empty
    DDR_A_RD_FALL_REN : out std_logic;                      -- DDR Port A Falling Data Enable
    DDR_A_RD_FALL_EF  : in  std_logic;                      -- DDR Port A Falling Data Empty Flag
    DDR_A_RD_FALL_AE  : in  std_logic;                      -- DDR Port A Falling Data Almost Empty
-- DDR PPC Interface for Port A
    PPC_TO_DDR_IF_A   : out std_logic_vector(130 downto 0); -- PowerPC to DDR Port A interface signals
    DDR_TO_PPC_IF_A   : in  std_logic_vector(77 downto 0)   -- DDR to PowerPC Port A interface signals
  );
end USER_AP;


architecture PPC_FFT_EX of USER_AP is

  -- Power PC Design
  component system
    port (
      sys_rst_n      : in  std_logic;
      sys_clk        : in  std_logic;
      Block_Ram_Rst  : in  std_logic;
      Block_Ram_Clk  : in  std_logic;
      Block_Ram_En   : in  std_logic;
      Block_Ram_Wen  : in  std_logic_vector(0 to 3);
      Block_Ram_Addr : in  std_logic_vector(0 to 31);
      Block_Ram_Din  : in  std_logic_vector(0 to 31);
      Block_Ram_Dout : out std_logic_vector(0 to 31);
      hpu_clock      : in  std_logic;
      hpu_start      : out std_logic;
      hpu_done       : in  std_logic
    );
  end component;

  -- CoreGen 256-point FFT
  component fft_2_1
    port (
      xn_re        : in  std_logic_vector(15 downto 0);
      xn_im        : in  std_logic_vector(15 downto 0);
      start        : in  std_logic;
      unload       : in  std_logic;
      nfft         : in  std_logic_vector(4 downto 0);
      nfft_we      : in  std_logic;
      fwd_inv      : in  std_logic;
      fwd_inv_we   : in  std_logic;
      scale_sch    : in  std_logic_vector(15 downto 0);
      scale_sch_we : in  std_logic;
      sclr         : in  std_logic;
      ce           : in  std_logic;
      clk          : in  std_logic;
      xk_re        : out std_logic_vector(15 downto 0);
      xk_im        : out std_logic_vector(15 downto 0);
      xn_index     : out std_logic_vector(7 downto 0);
      xk_index     : out std_logic_vector(7 downto 0);
      rfd          : out std_logic;
      busy         : out std_logic;
      dv           : out std_logic;
      edone        : out std_logic;
      done         : out std_logic;
      ovflo        : out std_logic
    );
  end component;

  -- CoreGen FIFO
  component fifo15x32
    port (
      din          : in  std_logic_vector(31 downto 0);
      wr_en        : in  std_logic;
      wr_clk       : in  std_logic;
      rd_en        : in  std_logic;
      rd_clk       : in  std_logic;
      ainit        : in  std_logic;
      dout         : out std_logic_vector(31 downto 0);
      full         : out std_logic;
      empty        : out std_logic;
      almost_full  : out std_logic;
      almost_empty : out std_logic
    );
  end component;

  component BUFG
    port(I : in  std_logic;
         O : out std_logic);
  end component;

  signal MCLK           : std_logic;  -- Main clock
  signal LEDcount       : unsigned(25 downto 0);

  -- Control signals for transfer from 15x32 FIFO to HERON FIFO-Write Interface
  signal RD_EN          : std_logic;
  signal EMPTY          : std_logic;

  -- PPC signals
  signal VCC            : std_logic;
  signal PPC_RESET      : std_logic;
  signal fft_start      : std_logic;
  signal fft_done       : std_logic;

  -- FFT input control signals
  signal start_rg1      : std_logic;
  signal start_rg2      : std_logic;
  signal start_setup    : std_logic;
  signal start          : std_logic;

  signal nfft_we        : std_logic;
  signal fwd_inv_we     : std_logic;
  signal scale_sch_we   : std_logic;

  signal nfft           : std_logic_vector(4 downto 0);
  signal fwd_inv        : std_logic;
  signal scale_sch      : std_logic_vector(15 downto 0);

  signal BRAM_DOUT_DL1  : std_logic_vector(31 downto 0);
  signal BRAM_DOUT_DL2  : std_logic_vector(31 downto 0);

  signal xn_re          : std_logic_vector(15 downto 0);
  signal xn_im          : std_logic_vector(15 downto 0);

  -- FFT output control signals
  signal dv             : std_logic;
  signal edone          : std_logic;
  signal unload         : std_logic;
  signal dval_done      : std_logic;

  signal FFT_DATA       : std_logic_vector(31 downto 0);

  -- DSOCM Block RAM control signals
  signal Block_Ram_Rst  : std_logic;
  signal Block_Ram_Clk  : std_logic;
  signal Block_Ram_En   : std_logic;
  signal Block_Ram_Wen  : std_logic_vector(3 downto 0);
  signal Block_Ram_Addr : std_logic_vector(31 downto 0);
  signal Block_Ram_Dout : std_logic_vector(31 downto 0);
  signal Block_Ram_Din  : std_logic_vector(31 downto 0);

  signal xk_re          : std_logic_vector(15 downto 0);
  signal xk_im          : std_logic_vector(15 downto 0);

  signal xn_index       : std_logic_vector(7 downto 0);
  signal xk_index       : std_logic_vector(7 downto 0);

  signal OSC3_CLK       : std_logic;

begin

  -- ==========================================================
  -- ==========================================================
  --       <<<<<<<   INSERT YOUR CODE HERE    >>>>>>>>>
  -- ==========================================================
  -- ==========================================================


  -------------------------------------------------------------
  -- Main clock to be used in this example

  iOSC3_BUFG : BUFG port map (I=>OSC3, O=>OSC3_CLK);

  MCLK <= OSC3_CLK;

  -------------------------------------------------------------
  -- Drive the UDP-RESET high (inactive)

  UDPRES <= '1';

  -------------------------------------------------------------
  -- Drive the MODULE HAS PROCESSOR signal high (inactive)

  MOD_HAS_PROC <= '1';

  -------------------------------------------------------------
  -- Drive unused signals

  UMI_OUT <= (others=>'0');
  UMI_EN  <= (others=>'1');

  -------------------------------------------------------------
  -- FIFO Clocks

  SRC_FCLK_RD <= '0';   -- clock source for READ FIFO
  SRC_FCLK_WR <= '0';   -- clock source for WRITE FIFO
  SRC_FCLK_G  <= MCLK;  -- global clock source for both FIFOs

  -------------------------------------------------------------
  -- Input FIFOs (not used in this example)

  INFIFO_READ_REQ <= (others=>'0');

  -------------------------------------------------------------
  -- Power PC design

  VCC <= '1';

  PPC_RESET <= NOT RESET;

  isystem : system
    port map (
      sys_rst_n      => PPC_RESET,
      sys_clk        => OSC3,
      Block_Ram_Rst  => Block_Ram_Rst,
      Block_Ram_Clk  => Block_Ram_Clk,
      Block_Ram_En   => Block_Ram_En,
      Block_Ram_Wen  => Block_Ram_Wen,
      Block_Ram_Addr => Block_Ram_Addr,
      Block_Ram_Din  => Block_Ram_Din,
      Block_Ram_Dout => Block_Ram_Dout,
      hpu_clock      => FCLK_G,
      hpu_start      => fft_start,
      hpu_done       => fft_done );

  -------------------------------------------------------------
  -- FFT 'hardware processsing unit'

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      start_rg1 <= '0';
      start_rg2 <= '0';
    elsif rising_edge(FCLK_G) then
      start_rg1 <= fft_start;
      start_rg2 <= start_rg1;
    end if;
  end process;

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      start_setup <= '0';
    elsif rising_edge(FCLK_G) then
      start_setup <= '0';
      if start_rg2='0' and start_rg1='1' then
        start_setup <= '1';
      end if;
    end if;
  end process;

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      start <= '0';
    elsif rising_edge(FCLK_G) then
      start <= start_setup;
    end if;
  end process;

  nfft_we      <= start_setup;
  fwd_inv_we   <= start_setup;
  scale_sch_we <= start_setup;

  -- Set to give 256 pt operation
  nfft <= "01000";

  -- Set to give a forward transform
  fwd_inv <= '1';

  -- Set scaling information
  scale_sch <= "0101010101010101";

  -- Take data from DSOCM BRAM and present it to the FFT unit
  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      BRAM_DOUT_DL1 <= (others=>'0');
      BRAM_DOUT_DL2 <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      BRAM_DOUT_DL1 <= Block_Ram_Dout;
      BRAM_DOUT_DL2 <= BRAM_DOUT_DL1;
    end if;
  end process;

  --Signed data input
  xn_re <= BRAM_DOUT_DL2(15 downto 0);
  xn_im <= BRAM_DOUT_DL2(31 downto 16);

  iFFT : fft_2_1
    port map (
      xn_re        => xn_re,
      xn_im        => xn_im,
      start        => start,
      unload       => unload,
      nfft         => nfft,
      nfft_we      => nfft_we,
      fwd_inv      => fwd_inv,
      fwd_inv_we   => fwd_inv_we,
      scale_sch    => scale_sch,
      scale_sch_we => scale_sch_we,
      sclr         => RESET,
      ce           => VCC,
      clk          => FCLK_G,
      xk_re        => xk_re,
      xk_im        => xk_im,
      xn_index     => xn_index,
      xk_index     => xk_index,
      rfd          => open,
      busy         => open,
      dv           => dv,
      edone        => edone,
      done         => open,
      ovflo        => open );

  -- Unload data from the FFT when it indicates it has completed
  unload <= edone;

  -- FFT Output data, real and imaginary placed in one 32-bit word
  FFT_DATA <= xk_im & xk_re;

  -- Wait for the FFT unloading stage to complete
  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      dval_done <= '0';
    elsif rising_edge(FCLK_G) then
      dval_done <= '0';
      if (dv='1' and xk_index="11111111") then
        dval_done <= '1';
      end if;
    end if;
  end process;

  -- Indicate to the Power PC that processing has completed
  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      fft_done <= '0';
    elsif rising_edge(FCLK_G) then
      fft_done <= '0';
      if dval_done='1' or (fft_done='1' and fft_start='1') then
        fft_done <= '1';
      end if;
    end if;
  end process;

  -------------------------------------------------------------
  -- DSOCM Block RAM Control

  Block_Ram_Rst <= '0';
  Block_Ram_Clk <= FCLK_G;
  Block_Ram_En  <= '1';

  Block_Ram_Wen(0) <= dv;
  Block_Ram_Wen(1) <= dv;
  Block_Ram_Wen(2) <= dv;
  Block_Ram_Wen(3) <= dv;

  Block_Ram_Din <= FFT_DATA;

  Block_Ram_Addr(9 downto 2) <= xn_index when dv='0' else xk_index;
  Block_Ram_Addr(10)         <= '0'      when dv='0' else '1';

  Block_Ram_Addr(1 downto 0)   <= (others=>'0');
  Block_Ram_Addr(31 downto 11) <= (others=>'0');

  -------------------------------------------------------------
  -- Output FIFOs

  iFIFO : fifo15x32
    port map (
      din          => FFT_DATA,
      wr_en        => dv,
      wr_clk       => FCLK_G,
      rd_en        => RD_EN,
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => OUTFIFO_D,
      full         => open,
      empty        => EMPTY,
      almost_full  => open,
      almost_empty => open );

  RD_EN <= OUTFIFO_READY(0) AND (NOT EMPTY);

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      OUTFIFO_WRITE(0) <= '0';
    elsif rising_edge(FCLK_G) then
      OUTFIFO_WRITE(0) <= RD_EN;
    end if;
  end process;

  OUTFIFO_WRITE(5 downto 1) <= (others=>'0');

  -------------------------------------------------------------
  -- DDR SDRAM (not used in this example)

  DDR_A_USER_CLK    <= '0';
  DDR_A_USER_RST    <= '0';

  DDR_A_WR_DATA     <= (others=>'0');
  DDR_A_WR_ADDR     <= (others=>'0');
  DDR_A_WR_ADDR_WEN <= '0';
  DDR_A_WR_RISE_WEN <= '0';
  DDR_A_WR_FALL_WEN <= '0';
  DDR_A_WR_DM_WEN   <= '0';

  DDR_A_RD_ADDR     <= (others=>'0');
  DDR_A_RD_ADDR_WEN <= '0';
  DDR_A_RD_RISE_REN <= '0';
  DDR_A_RD_FALL_REN <= '0';

  -------------------------------------------------------------
  -- PowerPC to DDR Interface (not used in this example)

  PPC_TO_DDR_IF_A <= (others=>'0');

  -------------------------------------------------------------
  -- Serial Bus (not used in this example)

  MSG_CLK       <= OSC3;  -- clock for Message Interface
  MSG_READY     <= '1';
  MSG_SEND_MSG  <= '0';
  MSG_CE        <= '0';
  MSG_SEND_ID   <= '0';
  MSG_LAST_BYTE <= '0';
  MSG_DOUT      <= (others=>'0');

  -------------------------------------------------------------
  -- Connector Digital IO (not used in this example)

  CONN_A_OUT <= (others=>'0');
  CONN_A_EN  <= (others=>'1');

  CONN_B_OUT <= (others=>'0');
  CONN_B_EN  <= (others=>'1');

  -------------------------------------------------------------
  -- FLASH memory interface (not used in this example)

  FLASH_ADDR <= (others=>'0');

  FLASH_OUT  <= (others=>'0');
  FLASH_EN   <= (others=>'1');

  FLASH_CE   <= '1';
  FLASH_WE   <= '1';
  FLASH_OE   <= '1';

  FLASH_RP   <= '0';
  FLASH_VPEN <= '0';

  -------------------------------------------------------------
  -- Led Flasher : dividing F_CLKG clock by 2**24

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      LEDcount <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      LEDcount <= LEDcount + 1;
    end if;
  end process;

  LED(4) <= LEDcount(24);

  -- Drive LEDs high (inactive)
  LED(3 downto 0) <= (others=>'1');

end PPC_FFT_EX;
