--------------------------------------------------------------------------------
-- FILE : USER_LOGIC.VHD
--------------------------------------------------------------------------------
--
-- This file is part of the FFT Example for the PowerPC.
--
--------------------------------------------------------------------------------
--
--  Date        : 17/09/2007
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : OPB to HPU interface
--
--------------------------------------------------------------------------------
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    17-09-07    First Written for EDK9.1, based on Xilinx
--                                     example code - OPB GPIO v3.01.a.
--
--------------------------------------------------------------------------------
--
--  Structure:
--
--       opb2hpu.vhd
--          \ opb_ipif_ssp1.vhd
--          \ user_logic.vhd
--
-------------------------------------------------------------------------------


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.std_logic_signed.all;
  use IEEE.std_logic_misc.all;


library Unisim;
  use Unisim.all;


entity user_logic is
  port (
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Clk   : in  std_logic;
    Bus2IP_CS    : in  std_logic;
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RdCE  : in  std_logic;
    Bus2IP_Reset : in  std_logic;
    Bus2IP_WrCE  : in  std_logic;
    IP2Bus_Ack   : out std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    HPU_CLOCK    : in  std_logic;
    HPU_START    : out std_logic;
    HPU_DONE     : in  std_logic;
    HPU_IRQ      : out std_logic
  );
end entity user_logic;


architecture RTL of user_logic is

  signal RdCE_rg          : std_logic;

  signal addr             : std_logic_vector(0 to 1);

  signal hpu_active       : std_logic;
  signal hpu_start_access : std_logic;
  signal hpu_en_access    : std_logic;
  signal hpu_clr_access   : std_logic;

  signal clear_access     : std_logic;

  signal hpu_int_en       : std_logic;
  signal hpu_int          : std_logic;

  signal start_rg         : std_logic;

  signal done_rg1         : std_logic;
  signal done_rg2         : std_logic;

begin

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      RdCE_rg <= Bus2IP_RdCE;
    end if;
  end process;

  IP2Bus_Ack <= Bus2IP_WrCE or RdCE_rg;

  IP2Bus_Data(0 to 28) <= (others=>'0');

  -- Signals read by the Power PC
  IP2Bus_Data(29) <= hpu_int;
  IP2Bus_Data(30) <= hpu_active;
  IP2Bus_Data(31) <= hpu_int_en;

  -- OPB Address - Big Endian
  addr <= Bus2IP_Addr(28 to 29);

  process (addr) is
  begin
    case addr is
      when "00"   => hpu_start_access <= '1'; hpu_en_access <= '0'; hpu_clr_access <= '0';
      when "01"   => hpu_start_access <= '0'; hpu_en_access <= '1'; hpu_clr_access <= '0';
      when "10"   => hpu_start_access <= '0'; hpu_en_access <= '0'; hpu_clr_access <= '1';
      when others => hpu_start_access <= '0'; hpu_en_access <= '0'; hpu_clr_access <= '0';
    end case;
  end process;

  -- HPU_ACTIVE becomes the start signal driven to the FPGA logic
  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      if Bus2IP_Reset='1' then
        hpu_active <= '0';
      else
        hpu_active <= '0';
        if (Bus2IP_CS='1' and Bus2IP_WrCE='1' and hpu_start_access='1') or
           (hpu_active='1' and done_rg2='0') then
          hpu_active <= '1';
        end if;
      end if;
    end if;
  end process;

  -- HPU_INT_EN is the hardware processing unit interrupt enable
  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      if Bus2IP_Reset='1' then
        hpu_int_en <= '0';
      else
        if (Bus2IP_CS='1' and Bus2IP_WrCE='1' and hpu_en_access='1') then
          hpu_int_en <= Bus2IP_Data(31);
        end if;
      end if;
    end if;
  end process;

  clear_access <= Bus2IP_CS AND Bus2IP_WrCE AND hpu_clr_access;

  -- Hardware processing unit interrupt signal
  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      if Bus2IP_Reset='1' then
        hpu_int <= '0';
      else
        hpu_int <= '0';
        if (hpu_active='1' and done_rg2='1' and hpu_int_en='1') or
         (hpu_int='1' and clear_access='0') then
          hpu_int <= '1';
        end if;
      end if;
    end if;
  end process;

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      if Bus2IP_Reset='1' then
        HPU_IRQ <= '0';
      else
        HPU_IRQ <= '0';
        if (hpu_int='1' and hpu_int_en='1') then
          HPU_IRQ <= '1';
        end if;
      end if;
    end if;
  end process;

  process(HPU_CLOCK) is
  begin
    if HPU_CLOCK'event and HPU_CLOCK='1' then
      start_rg  <= hpu_active;
      HPU_START <= start_rg;
    end if;
  end process;

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      done_rg1 <= HPU_DONE;
      done_rg2 <= done_rg1;
    end if;
  end process;

end architecture RTL;
